use faust_types::{ParamIndex, UI};
use std::collections::HashMap;

pub type ParameterMap = HashMap<String, ParamIndex>;

/// Helper struct that implements [`faust_types::UI`] to collect all
/// parameters of [`faust_types::FaustDsp`] into a HashMap
pub struct ParameterCollector {
    paths: ParameterMap,
}

impl ParameterCollector {
    pub fn new() -> Self {
        Self {
            paths: HashMap::new(),
        }
    }
    pub fn add_parameter(&mut self, label: &str, param: ParamIndex) {
        if let Some(_old_value) = self.paths.insert(label.to_string(), param) {
            panic!("Duplicate label: {label}");
        }
    }

    pub fn into_path_map(self) -> ParameterMap {
        self.paths
    }
}

impl UI<f32> for ParameterCollector {
    fn open_tab_box(&mut self, _label: &str) {}

    fn open_horizontal_box(&mut self, _label: &str) {}

    fn open_vertical_box(&mut self, _label: &str) {}

    fn close_box(&mut self) {}

    fn add_button(&mut self, label: &str, param: ParamIndex) {
        self.add_parameter(label, param);
    }

    fn add_check_button(&mut self, label: &str, param: ParamIndex) {
        self.add_parameter(label, param);
    }

    fn add_vertical_slider(
        &mut self,
        label: &str,
        param: ParamIndex,
        _init: f32,
        _min: f32,
        _max: f32,
        _step: f32,
    ) {
        self.add_parameter(label, param);
    }

    fn add_horizontal_slider(
        &mut self,
        label: &str,
        param: ParamIndex,
        _init: f32,
        _min: f32,
        _max: f32,
        _step: f32,
    ) {
        self.add_parameter(label, param);
    }

    fn add_num_entry(
        &mut self,
        label: &str,
        param: ParamIndex,
        _init: f32,
        _min: f32,
        _max: f32,
        _step: f32,
    ) {
        self.add_parameter(label, param);
    }

    fn add_horizontal_bargraph(&mut self, label: &str, param: ParamIndex, _min: f32, _max: f32) {
        self.add_parameter(label, param);
    }

    fn add_vertical_bargraph(&mut self, label: &str, param: ParamIndex, _min: f32, _max: f32) {
        self.add_parameter(label, param);
    }

    fn declare(&mut self, _param: Option<ParamIndex>, _key: &str, _value: &str) {}
}
