use eframe::{
    egui::{self, Sense, Ui},
    epaint::Color32,
};

use crate::{
    colors,
    paint::{alloc_slider, paint_slider},
};

/// Combined into one function (rather than two) to make it easier
/// for the borrow checker.
pub type GetSetValue<'a> = Box<dyn 'a + FnMut(Option<f32>) -> f32>;

pub struct Button<'a> {
    get_set_value: GetSetValue<'a>,
    text: &'a str,
}

/// A compact slider widget
impl<'a> Button<'a> {
    pub fn from_get_set(get_set_value: impl 'a + FnMut(Option<f32>) -> f32) -> Self {
        Self {
            get_set_value: Box::new(get_set_value),
            text: "NoLabel",
        }
    }
    pub fn text(mut self, text: &'a str) -> Self {
        self.text = text;
        self
    }
    fn value(&mut self) -> f32 {
        (self.get_set_value)(None)
    }

    fn set_value(&mut self, value: f32) {
        (self.get_set_value)(Some(value));
    }
}

impl egui::Widget for Button<'_> {
    fn ui(mut self, ui: &mut Ui) -> egui::Response {
        let response = ui.allocate_response(alloc_slider(), Sense::click());
        let rect = &response.rect;

        if response.clicked() {
            let new_value = if self.value() == 1.0 { 0.0 } else { 1.0 };
            self.set_value(new_value);
        }

        // Paint
        if ui.is_rect_visible(*rect) {
            let value = self.value();
            paint_slider(
                ui,
                response.rect,
                1.0,
                if value == 0.0 {
                    Color32::WHITE
                } else {
                    colors::PINK
                },
                &format!(
                    "{}: {}",
                    self.text,
                    if value == 0.0 { "Off" } else { " On" }
                ),
            );
        }
        response
    }
}
