use eframe::{
    egui::{self, Sense, Ui},
    emath::Align2,
    epaint::{Color32, FontId, Vec2},
};

pub const FONT_SIZE: f32 = 20.0;
pub struct Label<'a> {
    text: &'a str,
}

impl<'a> Label<'a> {
    pub fn new(text: &'a str) -> Self {
        Self { text }
    }
}

impl egui::Widget for Label<'_> {
    fn ui(self, ui: &mut Ui) -> egui::Response {
        let response = ui.allocate_response(
            Vec2 {
                x: crate::paint::WIDTH,
                y: FONT_SIZE,
            },
            Sense::click(),
        );
        let rect = response.rect;
        // Paint
        if ui.is_rect_visible(rect) {
            let font_id = FontId {
                size: FONT_SIZE,
                ..Default::default()
            };
            ui.painter().text(
                rect.left_center(),
                Align2::LEFT_CENTER,
                self.text,
                font_id,
                Color32::BLACK,
            );
        }
        response
    }
}
