use eframe::{
    egui::Ui,
    emath::Align2,
    epaint::{Color32, FontId, Rect, Rounding, Vec2},
};

pub const BORDER: f32 = 1.0;
pub const WIDTH: f32 = 150.0;
pub const HEIGHT: f32 = 20.0;

pub fn alloc_slider() -> Vec2 {
    Vec2 {
        y: HEIGHT,
        x: WIDTH,
    }
}

pub fn paint_slider(ui: &mut Ui, rect: Rect, progress: f32, progress_color: Color32, text: &str) {
    // Outer rectangle
    ui.painter()
        .rect_filled(rect, Rounding::default(), Color32::BLACK);
    // Inner white background
    ui.painter()
        .rect_filled(rect.shrink(BORDER), Rounding::default(), Color32::WHITE);

    // Progress rectangle
    let mut progress_rect = rect.shrink(BORDER);
    progress_rect.set_width(progress_rect.width() * progress);
    ui.painter()
        .rect_filled(progress_rect, Rounding::default(), progress_color);

    // Text
    let font_id = FontId {
        size: 13.0,
        ..Default::default()
    };
    ui.painter().text(
        rect.center(),
        Align2::CENTER_CENTER,
        text,
        font_id,
        Color32::BLACK,
    );
}
