use eframe::{
    egui::{self, Sense, Ui},
    emath::remap_clamp,
};

use crate::{
    colors,
    paint::{alloc_slider, paint_slider},
};

/// Combined into one function (rather than two) to make it easier
/// for the borrow checker.
pub type GetSetValue<'a> = Box<dyn 'a + FnMut(Option<f32>) -> f32>;

pub struct Slider<'a> {
    get_set_value: GetSetValue<'a>,
    text: &'a str,
}

/// A compact slider widget
impl<'a> Slider<'a> {
    pub fn from_get_set(get_set_value: impl 'a + FnMut(Option<f32>) -> f32) -> Self {
        Self {
            get_set_value: Box::new(get_set_value),
            text: "NoLabel",
        }
    }
    pub fn text(mut self, text: &'a str) -> Self {
        self.text = text;
        self
    }

    fn value(&mut self) -> f32 {
        (self.get_set_value)(None)
    }

    fn set_value(&mut self, value: f32) {
        (self.get_set_value)(Some(value));
    }
}

impl egui::Widget for Slider<'_> {
    fn ui(mut self, ui: &mut Ui) -> egui::Response {
        let response = ui.allocate_response(alloc_slider(), Sense::drag());
        let rect = &response.rect;

        if let Some(pointer_position_2d) = response.interact_pointer_pos() {
            let position = pointer_position_2d.x;
            let position_range = rect.left()..=rect.right();
            let new_value = remap_clamp(position, position_range, 0.0..=1.0);
            self.set_value(new_value);
        }

        // Paint
        if ui.is_rect_visible(*rect) {
            let value = self.value();
            paint_slider(
                ui,
                response.rect,
                self.value(),
                colors::LIGHT_GRAY,
                &format!("{}: {:.02}", self.text, value),
            );
        }
        response
    }
}
